package sockprog.example1_hello;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public final class MessageServer {
    public static void main(String[] args) { // [1]
        if(args.length < 1) {
            System.err.println("parameter not enough");
            System.exit(-1);
        }
        int port = Integer.parseInt(args[0]);
        
        try {
            MessageServer server = new MessageServer(port);  
            server.start();
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }

    private final ServerSocket serverSocket;
    private final Thread thread;

    public MessageServer(int port) throws IOException { // [2]
        serverSocket = new ServerSocket(port);   // [3]

        thread = new Thread(new Runnable() { // [4]
            public void run() {
                service();
            }
        });
    }
    
    public void start() { // [5]
        thread.start();
    }
    
    private void service() { // [6]
        while(true) {
            try {
                Socket socket = serverSocket.accept(); // [7]
                new EchoClientHandler(socket).run(); // [8]
            } catch(IOException e) {
                return; // [9]
            }
        }
    }
    
    public void shutDown() // [10] 
    throws IOException, InterruptedException {
        serverSocket.close();
        thread.join();
    }

}
