package sockprog.example2_reuse;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.Socket;

final class EchoClientHandler implements Runnable {
    private final Socket socket;

    EchoClientHandler(Socket socket) {
        this.socket = socket; 
    }
    
    public void run() {
        try {
            handleClient();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                socket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void handleClient() throws IOException {
        // readerの準備
        InputStream in = socket.getInputStream();
        InputStreamReader inReader = 
            new InputStreamReader(in, ProtocolConstants.ENCODING);
        BufferedReader reader = new BufferedReader(inReader);

        // writerの準備
        OutputStream out = socket.getOutputStream();
        OutputStreamWriter outWriter = 
            new OutputStreamWriter(out, ProtocolConstants.ENCODING);
        Writer writer = new BufferedWriter(outWriter);

        // 読み込みが終わるまでEchoループ
        for(String message = reader.readLine();
        message != null; message = reader.readLine()) {
            writer.write(message);
            writer.write(ProtocolConstants.LINE_SEPARATOR);
            writer.flush();
            System.out.println(message);
        }
    }
}
