package sockprog.example3_shutdown;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;

public final class MessageServer {
    public static void main(String[] args) {
        if(args.length < 1) {
            System.err.println("parameter not enough");
            System.exit(-1);
        }
        int port = Integer.parseInt(args[0]);
        
        try {
            MessageServer server = new MessageServer(port);  
            server.start();
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }

    private final ServerSocket serverSocket;
    private final Thread thread;

    public MessageServer(int port) throws IOException {
        serverSocket = new ServerSocket();
        serverSocket.setReuseAddress(true);
        serverSocket.bind(new InetSocketAddress(port));

        thread = new Thread(new Runnable() {
            public void run() {
                service();
            }
        });
    }
    
    public void start() {
        thread.start();
    }
    
    private void service() {
        while(true) {
            try {
                Socket socket = serverSocket.accept();
                new EchoClientHandler(socket).run();
            } catch(IOException e) {
                return;
            }
        }
    }
    
    public void shutDown()
    throws IOException, InterruptedException {
        serverSocket.close();
        thread.join();
    }

}
