package sockprog.example5_buffer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.Socket;
import java.net.SocketTimeoutException;

final class EchoClientHandler implements Runnable {
    private final Socket socket;
    private final int timeout;
    private final int bufferSize;

    EchoClientHandler(Socket socket, int timeout, int bufferSize) {
        this.socket = socket;
        this.timeout = timeout;
        this.bufferSize = bufferSize;
    }

    public void run() {
        try {
            handleClient();
        } catch (SocketTimeoutException e) {
            // タイムアウトの際の処理を行う
            System.err.println("timeout");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                socket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void handleClient() throws IOException {
        // 送信バッファサイズの設定
        socket.setSendBufferSize(bufferSize);

        // タイムアウトの設定
        socket.setSoTimeout(timeout);

        // readerの準備
        InputStream in = socket.getInputStream();
        InputStreamReader inReader =
            new InputStreamReader(in, ProtocolConstants.ENCODING);
        BufferedReader reader = new BufferedReader(inReader, bufferSize);

        // writerの準備
        OutputStream out = socket.getOutputStream();
        OutputStreamWriter outWriter =
            new OutputStreamWriter(out, ProtocolConstants.ENCODING);
        Writer writer = new BufferedWriter(outWriter, bufferSize);

        // 読み込みが終わるまでEchoループ
        for(String message = reader.readLine();
        message != null; message = reader.readLine()) {
            writer.write(message);
            writer.write(ProtocolConstants.LINE_SEPARATOR);
            writer.flush();
            System.out.println(message);
        }
    }
}
