import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.UnknownHostException;

/**
 * UDPマルチキャスト受信プログラム： 送られてきたマルチキャストデータを受信する。
 * 
 * @author T.Ishiguro
 * 
 */
public class UDPMulticastReceiver {

	private final int BUFFER_SIZE = 64;
	private InetAddress inetAddress;
	private int port;

	public UDPMulticastReceiver(String address, int port)
			throws UnknownHostException, IOException {
		this.inetAddress = InetAddress.getByName(address);
		this.port = port;
	}

	public void doJob() {
		MulticastSocket multicastSocket = null;
		try {
			multicastSocket = new MulticastSocket(port);
			multicastSocket.joinGroup(inetAddress);
			byte[] receiveBuffer = new byte[BUFFER_SIZE];
			DatagramPacket packet = new DatagramPacket(receiveBuffer,
					receiveBuffer.length);
			while (true) {
				multicastSocket.receive(packet);
				System.out.println(new String(packet.getData(), 0, packet
						.getLength()) + " from " + packet.getAddress()
						+ ":" + packet.getPort());
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		} finally {
			try {
				multicastSocket.leaveGroup(inetAddress);
			} catch (IOException ex) {
				ex.printStackTrace();
			}
			multicastSocket.close();
		}
	}

	public static void main(String[] args) {
		final String MULTICAST_ADDRESS = "239.255.10.10"; // IPv4
		final int PORT = 10101;

		try {
			new UDPMulticastReceiver(MULTICAST_ADDRESS, PORT).doJob();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}
