import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

/**
 * UDPマルチキャスト送信プログラム： 自分の名前を5秒おきにマルチキャスト送信する。
 * 
 * @author T.Ishiguro
 * 
 */
public class UDPMulticastSender {

	private final String NAME = "T.Ishiguro";
	private final int INTERVAL = 1000 * 5; // 5秒
	private InetAddress inetAddress;
	private int port;

	public UDPMulticastSender(String address, int port)
			throws UnknownHostException, SocketException {
		this.inetAddress = InetAddress.getByName(address);
		this.port = port;
	}

	public void doJob() {
		DatagramSocket datagramSocket = null;
		try {
			datagramSocket = new DatagramSocket();
			int i = 1; // 送信データのシーケンス番号
			while (true) {
				byte[] sendData = String.format("%04d " + NAME, i++).getBytes();
				DatagramPacket packet = new DatagramPacket(sendData,
						sendData.length, inetAddress, port);
				datagramSocket.send(packet);
				System.out.println(new String(packet.getData()) + " to "
						+ packet.getAddress() + ":" + packet.getPort());
				Thread.sleep(INTERVAL);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			datagramSocket.close();
		}
	}

	public static void main(String[] args) {
		final String MULTICAST_ADDRESS = "239.255.10.10"; // IPv4
		final int PORT = 10101;

		try {
			new UDPMulticastSender(MULTICAST_ADDRESS, PORT).doJob();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}
