%%% -*- coding: utf-8 -*-
%%% ↑Emacsで文字コードを強制するときのタグです。

%%% モジュール名はファイルのベース名と一致させます。
%%% モジュール名には、文字列ではなくアトム(≒シンボル)を指定します。
%%% 小文字から始まる名前がアトムになります。
-module(receiver).

-export([main/1]).

%%% includeはマクロプロセッサで処理されます。escriptでは使えないようです。
-include("mcconfig.hrl").

%% 受信用プロセスのメインループです。
loop(Socket) ->
    receive
        {udp, Socket, Addr, Port, Data} ->
	    %% ソケットから受信したデータを整形して表示用プロセスに送信
	    %% しています。
            Msg = erlang:binary_to_list(Data),
            {Cnt, Rest} = string:to_integer(Msg),
            Name = string:strip(Rest),
            display:send(Cnt, Name, Addr, Port),
            loop(Socket)
    end.

%% 受信用プロセスの開始点です。起動時にコマンドラインから指定します。
main(Arg) ->
    if Arg == ['6'] -> % IPv6受信
	    %% inet:setoptsではIPV6_JOIN_GROUP相当を指定できないため、
	    %% IPv6受信は未サポートです。
            erlang:error("IPv6 multicast receiver is not supported."),
            Socket = error;
       true -> % IPv4受信
	    Opts = [binary, inet, {reuseaddr, true}],
            {ok, Socket} = gen_udp:open(?MULTICAST_PORT, Opts),
            {ok, Ip} = inet:getaddr(?MULTICAST_ADDR4, inet),
            {ok, Zero} = inet:getaddr("0.0.0.0", inet),
	    %% inet:setopts()とCのソケットAPIであるsetsockopt()との
	    %% 対応付けは、erts/emulator/drivers/common/inet_drv.c
	    %% に記述されています。
            inet:setopts(Socket, [{add_membership, {Ip, Zero}}]),
            io:format("Mode: IPv4~n~n", [])
    end,
    display:start(), % 表示処理を別プロセスで起動します。
    loop(Socket).
