package mywebserver;

import static java.net.HttpURLConnection.HTTP_BAD_REQUEST;
import static java.net.HttpURLConnection.HTTP_INTERNAL_ERROR;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import mywebserver.Request.BadRequestException;

/**
 * MyWebServerNXB
 */
public class MyWebServer {

    /**
     * T[o\PbgB
     */
    private final ServerSocket socket;

    /**
     * Xbhv[B
     */
    private final ExecutorService threads;

    /**
     * HTTPnhB
     */
    private final HttpHandler httpHandler;

    /**
     * MyWebServer\z܂B<br>
     * T[o\PbgEXbhv[EHTTPnh܂B
     * 
     * @param port
     *            HTTPT[õ|[gԍB
     * @param nThreads
     *            XbȟB
     * @param path
     *            hLg[g̃pXB
     * @param directoryIndex
     *            fBNgCfbNX̃t@CB
     */
    public MyWebServer(int port, int nThreads, String path,
            String directoryIndex) throws IOException {
        socket = new ServerSocket();
        socket.setReuseAddress(true);
        socket.bind(new InetSocketAddress(port));
        threads = Executors.newFixedThreadPool(nThreads);
        httpHandler = new HttpHandler(path, directoryIndex);
    }

    /**
     * NCAgNGXg̑҂󂯂Jn܂B<br>
     * NCAgƂ̃RlNVmAXɂ̃RlNVXbhɊ蓖Ă܂B
     */
    public void listen() {
        while (true) {
            try {
                final Socket conn = socket.accept();
                threads.execute(new Runnable() {
                    public void run() {
                        handle(conn);
                    }
                });
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 蓖ĂꂽRlNVnhO܂B<br>
     * {@link java.net.ServerSocket#accept()}ĂяoXbhƂ͕ʃXbhœ삵܂B<br>
     * X|XIuWFNgENGXgIuWFNg\zAHTTPnhɏϏ܂B<br>
     * NGXgwb_̉͂Ɏsꍇ Status-Code 400 Bad Request NCAgɑM܂B<br>
     * sȗOꍇ Status-Code 500 Internal Server Error NCAgɑM܂B
     * 
     * @param conn
     *            RlNVB
     */
    private void handle(Socket conn) {
        try {
            Response res = new Response(conn.getOutputStream());
            try {
                Request req = new Request(conn.getInputStream());
                httpHandler.handle(req, res);
            } catch (BadRequestException e) {
                res.sendResponseHeader(HTTP_BAD_REQUEST);
            } catch (Exception e) {
                res.sendResponseHeader(HTTP_INTERNAL_ERROR);
                throw e;
            } finally {
                if (!conn.isClosed()) {
                    conn.close();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * MyWebServer̃Gg[|CgB
     * 
     * @param args
     *            vOB
     * @throws IOException
     *             œo͗OB
     */
    public static void main(String[] args) throws IOException {
        MyWebServer server = new MyWebServer(8081, 150, "C:\\work\\webroot\\",
                "index.html");
        server.listen();
    }

}
